
/* Printing over the ARM Cortex debug interface using openocd/gcc/gdb on Linux
 * This program was developed using an STM32F042 Nucleo board
 * IMPORTANT you must execute this in your arm-none-eabi-gdb session 
 * before running the program:
  monitor arm semihosting enable
 * if you don't do this, the processor halts on the breakpoint
 
 * Typically the sequence of debug commands are as following (inside gdb)
  
  target remote :3333
  monitor arm semihosting enable
  monitor reset halt
  load main.elf
  monitor reset

*/


#include "stm32f042.h"

void delay(int);

void delay(int dly)
{
  while( dly--);
}

void initClock()
{
// This is potentially a dangerous function as it could
// result in a system with an invalid clock signal - result: a stuck system
        // Set the PLL up
        // First ensure PLL is disabled
        RCC_CR &= ~BIT24;
        while( (RCC_CR & BIT25)); // wait for PLL ready to be cleared
        // set PLL multiplier to 12 (yielding 48MHz)
  // Warning here: if system clock is greater than 24MHz then wait-state(s) need to be
        // inserted into Flash memory interface
        FLASH_ACR |= BIT0;
        FLASH_ACR &=~(BIT2 | BIT1);

        // Turn on FLASH prefetch buffer
        FLASH_ACR |= BIT4;
        RCC_CFGR &= ~(BIT21 | BIT20 | BIT19 | BIT18);
        RCC_CFGR |= (BIT21 | BIT19 ); 

        // Need to limit ADC clock to below 14MHz so will change ADC prescaler to 4
        RCC_CFGR |= BIT14;

// Do the following to push HSI clock out on PA8 (MCO)
// for measurement purposes.  Should be 8MHz or thereabouts (verified with oscilloscope)
/*
        RCC_CFGR |= ( BIT26 | BIT24 );
        RCC_AHBENR |= BIT17;
        GPIOA_MODER |= BIT17;
*/

        // and turn the PLL back on again
        RCC_CR |= BIT24;        
        // set PLL as system clock source 
        RCC_CFGR |= BIT1;
}

void configPins()
{
	// Power up PORTB
	RCC_AHBENR |= BIT18;
	
}	

int main()
{
	unsigned *BadMemoryAddress;
	//initClock();
	unsigned i=0;
	configPins(); 
	GPIOB_MODER |= BIT6; // make bit3  an output
	GPIOB_MODER &= ~BIT7; // make bit3  an output
	// Point at a known bad memory address (reserved/unconnected space)
	BadMemoryAddress = (unsigned *)0xfffffffc;
	while(1)
	{
		
		GPIOB_ODR |= BIT3;
		delay(100000);
		GPIOB_ODR &= ~BIT3;
		delay(100000);
		// Generate a hard fault (exception number 3)
		// by writing to a bad memory address
		*BadMemoryAddress = 42;
	} 
	return 0;
}








